/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxiviewprintout.cpp
// Author:      Daniel Nell
// Created:     2005/10/18
// Copyright:   (c) Daniel Nell
// Licence:     wxWindows license
// Modified by:
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>
#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

// Include additional wxWidgets headers.
#include <wx/print.h>

// Include private wxIView headers.
#include "wxiviewpaint.h"
#include "wxiviewprintout.h"


//////////////////////////////////////////////////////////
// Print out object
//
wxIViewPrintout::wxIViewPrintout( wxIViewPaintBase *poPaint,
                                  const wxString& strTitle,
                                  bool bFitToPage,
                                  int nResolutionX,
                                  int nResolutionY )
  : wxPrintout( strTitle )
{
    // Initialize member variables.
    m_poPaint= poPaint;
    m_bFitToPage= bFitToPage;
    m_nResolutionX= nResolutionX;
    m_nResolutionY= nResolutionY;
}

void wxIViewPrintout::GetPageInfo( int *nMinPage, int *nMaxPage,
                                   int *nSelPageFrom, int *nSelPageTo )
{
    // Return 1 for the minimal page number and the correct number of pages
    // returned by wxIViewPaintBase::GetMaxPage() (if a valid wxIViewPaintBase object
    // is connnected to the wxIViewPrint object).
    (*nMinPage)= (*nMaxPage)= (*nSelPageFrom)= (*nSelPageTo)= 1;
    if( m_poPaint )
    {
        (*nMaxPage)= (*nSelPageTo)= m_poPaint->GetMaxPage();
    }
}

bool wxIViewPrintout::HasPage( int nPage )
{
    int nMinPage;
    int nMaxPage;
    int nDummy;

    GetPageInfo( &nMinPage, &nMaxPage, &nDummy, &nDummy );
    return ( nPage >= nMinPage) && ( nPage <= nMaxPage );
}

bool wxIViewPrintout::OnPrintPage( int nPage )
{
    // Check validity of the wxIViewPaintBase object (normaly the wxIViewFrame object).
    if( !m_poPaint )
    {
        // Signal error.
        return false;
    }

    // Get a device context.
    wxDC *oDc = GetDC();

    // Check validity of the device context.
    if( !oDc )
    {
        // Signal error.
        return false;
    }

    // Get printer resolution ...
    int nPrinterPpiX;
    int nPrinterPpiY;

    GetPPIPrinter( &nPrinterPpiX, &nPrinterPpiY );

    // ... and adjust scalation factor using image resolution.
    //
    // NOTE: We asume a fixed image resolution of 180 PPI in X and Y direction
    //       until there is a possibility to get the image resoltuion from
    //       image (EXIF/IPTC) itself. You can change this value by changing
    //       nResolutionX and wxivIMAGEPPIY in file wxIViewPaint.h.
    //
    double nScaleFactorX;
    double nScaleFactorY;

    if( IsPreview() )
    {
        int nPageWidth;
        int nPageHeight;
        int nDcWidth;
        int nDcHeight;

        oDc->GetSize( &nDcWidth, &nDcHeight );
        GetPageSizePixels( &nPageWidth, &nPageHeight );
        nScaleFactorX=   (double)nPrinterPpiX / (double)m_nResolutionX
                       * (double)nDcWidth     / (double)nPageWidth;
        nScaleFactorY=   (double)nPrinterPpiY / (double)m_nResolutionY
                       * (double)nDcHeight    / (double)nPageHeight;
    }
    else
    {
        nScaleFactorX= (double)nPrinterPpiX / (double)m_nResolutionX;
        nScaleFactorY= (double)nPrinterPpiY / (double)m_nResolutionY;
    }

    // Scale image to fit full page size if given.
    if( m_bFitToPage )
    {

        int nImageWidthMM;
        int nImageHeightMM;
        int nPageWidthMM;
        int nPageHeightMM;

        if( m_poPaint->GetPageSizeMM( &nImageWidthMM, &nImageHeightMM ) )
        {
            GetPageSizeMM( &nPageWidthMM, &nPageHeightMM );

            double nFitToPageScaleFactorX=   (double)nPageWidthMM
                                           / (double)nImageWidthMM;
            double nFitToPageScaleFactorY=   (double)nPageHeightMM
                                           / (double)nImageHeightMM;

            if( nFitToPageScaleFactorX > nFitToPageScaleFactorY )
            {
                nFitToPageScaleFactorX= nFitToPageScaleFactorY;
            }
            nScaleFactorX *= nFitToPageScaleFactorX;
            nScaleFactorY *= nFitToPageScaleFactorX;
        }
    }

    // Set device origin to (0, 0).
    oDc->SetDeviceOrigin( 0, 0 );

    // Set scale factor.
    oDc->SetUserScale( nScaleFactorX, nScaleFactorY );

    // Set printer page...
    if( !m_poPaint->GotoPage( nPage ) )
    {
        return false;
    }

    // ... and paint it.
    return m_poPaint->Paint( *oDc );
}


//////////////////////////////////////////////////////////
// Print preview frame object
//
void wxIViewPreviewFrame::CreateControlBar()
{
    // Note: This code is based on the default implementation of
    //       wxPreviewFrame::CreateControlBar() which you can find in file
    //       ${WXWIN}/src/common.prntbase.cpp .
    //
    // Default controls. In this implemention only the zoom control is shown
    // by default.
    long nButtons= wxPREVIEW_ZOOM;

    // Add the first/previous/next/last page buttons if there is a wxPrintOut object
    // associated and if wxPrintOut::GetPageInfo() returns more than one page.
    if( m_printPreview->GetPrintout() )
    {
        int nMinPage;
        int nMaxPage;
        int nDummy;

        m_printPreview->GetPrintout()->GetPageInfo( &nMinPage, &nMaxPage,
                                                    &nDummy, &nDummy );

        // Check if there is more than one page.
        if( nMaxPage - nMinPage > 0 )
        {
            // Add buttons for 1st/previous/next/last page.
            nButtons |=   wxPREVIEW_FIRST | wxPREVIEW_PREVIOUS
                        | wxPREVIEW_NEXT  | wxPREVIEW_LAST
                        | wxPREVIEW_GOTO;
        }
    }

    // Add the 'Print' button if there is a wxPrintOut object for printing associated.
    if( m_printPreview->GetPrintoutForPrinting() )
    {
        nButtons |= wxPREVIEW_PRINT;
    }

    // Create a new preview tool bar...
    m_controlBar = new wxPreviewControlBar( m_printPreview, nButtons, this,
                                            wxPoint( 0, 0 ), wxSize( 400, 40 ) );

    // ... and update buttons.
    m_controlBar->CreateButtons();
}
